package com.starbuds.server.service.feed.impl;

import com.starbuds.server.common.constant.feed.FeedCreateBusinessEnum;
import com.starbuds.server.common.constant.feed.FeedStatusEnum;
import com.starbuds.server.common.constant.feed.FeedTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.db.FeedPhotoWallDO;
import com.starbuds.server.common.pojo.feed.FeedPhotoWallVO;
import com.starbuds.server.common.pojo.mq.QueueFeedCreateMsg;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.feed.FeedPohtoWallService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.tag.feed.UserTag;
import com.starbuds.server.service.feed.dao.FeedPhotoWallDAO;
import com.starbuds.server.service.feed.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

import static com.starbuds.server.common.constant.feed.FeedStatusEnum.*;

/**
 * 内容服务-照片墙
 *
 * @author aoe
 * @date 2021/1/29
 */
@Service(version = "1.0.0")
@Slf4j
public class FeedPohtoWallServiceImpl implements FeedPohtoWallService {

    @Reference(version = "1.0.0")
    private UserService userService;

    @Resource
    private FeedPhotoWallDAO feedPhotoWallDAO;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RocketProducer rocketProducer;

    @Override
    public int getPhotoWallCount(Long userId) {
        return feedPhotoWallDAO.selectCountByUserId(userId);
    }

    /**
     * 创建照片墙
     *
     * @param fpw
     * @param channelId 渠道id（统计时使用）
     * @return 主键
     */
    @Override
    public FeedPhotoWallVO createPhotoWall(FeedPhotoWallDO fpw, Long channelId) {
        Long id = identityService.getId();
        fpw.setFeedPhotoWallId(id);
        fpw.setStatus(FeedStatusEnum.AUDIT_ROBOT.value());
        fpw.setAdminId(0L);
        fpw.setVersion(0L);
        fpw.setCreateTime(System.currentTimeMillis());
        fpw.setUpdateTime(fpw.getCreateTime());

        if (feedPhotoWallDAO.insert(fpw) != 1) {
            return null;
        }

        // 创建照片墙
        QueueFeedCreateMsg createMsg = new QueueFeedCreateMsg();
        createMsg.setFeedId(id);
        createMsg.setUrl(fpw.getUrl());
        createMsg.setFeedType(FeedTypeEnum.PHOTO.value());
        createMsg.setChannelId(channelId);
        createMsg.setBusinessEnum(FeedCreateBusinessEnum.PHOTO_WALL);
        createMsg.setUserId(fpw.getUserId());
        boolean sendResult = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.FEED_CREATE, createMsg.serialize());

        log.info("发送创建内容通知 sendResult={} msg={}", sendResult, createMsg);

        FeedPhotoWallVO vo = new FeedPhotoWallVO();
        BeanUtils.copyProperties(fpw, vo);
        vo.setStatus(AuditStatusEnum.WAIT.getCode());

        return vo;
    }

    @Override
    public List<FeedPhotoWallVO> getPhotoWallList(Long userId, Integer status) {
        List<FeedPhotoWallVO> list = feedPhotoWallDAO.selectList(userId, status);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        list.forEach(vo -> {
            FeedStatusEnum feedStatus = FeedStatusEnum.toEnum(vo.getStatus());
            vo.setStatus(AuditStatusEnum.WAIT.getCode());
            if (feedStatus == PASSED) {
                vo.setStatus(AuditStatusEnum.PASS.getCode());
            } else if (feedStatus == REFUSE_ROBOT || feedStatus == REFUSE_ARTIFICIAL) {
                vo.setStatus(AuditStatusEnum.DENY.getCode());
            }
        });
        return list;
    }

    /**
     * 删除照片墙
     *
     * @param feedPhotoWallId
     * @param userId
     * @return
     */
    @Override
    public boolean deletePhotoWall(Long feedPhotoWallId, Long userId) {
        return feedPhotoWallDAO.delete(userId, feedPhotoWallId, System.currentTimeMillis()) == 1;
    }

    /**
     * 设置用户信息
     *
     * @param list
     */
    @Override
    public <T> T setUserInfo(List<? extends UserTag> list) {
        if (CollectionUtils.isEmpty(list)) {
            return (T) list;
        }

        Map<Long, UserCardVO> userMap = getUserMap(list);
        for (UserTag tag : list) {
            if (tag == null) {
                continue;
            }
            Long userId = tag.getUserId();
            if (userId == null) {
                continue;
            }
            UserCardVO user = userMap.get(userId);
            if (user == null) {
                continue;
            }

            tag.setUserNo(user.getUserNo());
            tag.setUserName(user.getUserName());
            tag.setUserAvatar(user.getUserAvatar());
        }

        return (T) list;
    }

    @Override
    public FeedPhotoWallDO getFeedPhotoWall(Long feedPhotoWallId) {
        return feedPhotoWallDAO.selectById(feedPhotoWallId);
    }

    @Override
    public boolean exist(Long feedPhotoWallId) {
        return feedPhotoWallDAO.selectCountById(feedPhotoWallId) > 0;
    }

    /**
     * APP用户信息
     *
     * @param list UserTag 列表
     * @return key：用户Id value：用户信息
     */
    private Map<Long, UserCardVO> getUserMap(List<? extends UserTag> list) {
        Set<Long> userIds = getUserIds(list);
        return getUserMap(userIds);
    }

    /**
     * APP用户信息
     *
     * @param userIds APP用户Id列表
     * @return key：用户Id value：用户信息
     */
    private Map<Long, UserCardVO> getUserMap(Set<Long> userIds) {
        Map<Long, UserCardVO> userMap = new HashMap<>(userIds.size());
        if (userIds.isEmpty()) {
            return userMap;
        }
        List<UserCardVO> userList = userService.getNameAndAvatarByIds(new ArrayList<>(userIds));
        if (CollectionUtils.isEmpty(userList)) {
            return userMap;
        }

        for (UserCardVO user : userList) {
            if (user == null || user.getUserId() == null) {
                continue;
            }
            userMap.put(user.getUserId(), user);
        }

        return userMap;
    }

    private Set<Long> getUserIds(List<? extends UserTag> list) {
        Set<Long> userIds = new HashSet<>(list.size());
        for (UserTag tag : list) {
            if (tag == null || tag.getUserId() == null) {
                continue;
            }
            userIds.add(tag.getUserId());
        }

        return userIds;
    }
}
