package com.starbuds.server.service.feed.provider;

import com.starbuds.server.common.constant.statistics.StatRewardFeedGiftTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatRewardFeedGiftDO;
import com.starbuds.server.service.feed.dao.SendGiftRewardFeedSnapshotDAO;
import com.starbuds.server.service.feed.dao.StatRewardFeedGiftDAO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 打赏动态
 *
 * @author aoe
 * @date 2021/11/17
 */
@Slf4j
@Component
public class FeedGiftRewardProvider {

    @Resource
    private StatRewardFeedGiftDAO statRewardFeedGiftDAO;

    @Resource
    private SendGiftRewardFeedSnapshotDAO sendGiftRewardFeedSnapshotDAO;

    /**
     * 打赏动态Map
     *
     * @param feedIds 动态id集合
     * @return 动态id:打赏显示数额
     */
    public Map<Long, String> giftRewardMap(Collection<Long> feedIds) {
        Map<Long, Long> statMap = statRewardMap(feedIds);
        Map<Long, String> map = new HashMap<>();
        feedIds.stream()
                .filter(Objects::nonNull)
                .forEach(feedId -> {
                    Long total = statMap.get(feedId);
                    if (total == null) {
                        return;
                    }
                    map.put(feedId, getRewardStr(total));
                });
        return map;
    }

    /**
     * 已打赏的动态
     *
     * @param userId  用户id
     * @param feedIds 动态id集合
     * @return 已打赏的动态id集合
     */
    public Set<Long> rewardedMap(Long userId, Collection<Long> feedIds) {
        if (userId == null || CollectionUtils.isEmpty(feedIds)) {
            return Collections.emptySet();
        }

        List<Long> ids = sendGiftRewardFeedSnapshotDAO.selectFeedIds(userId, feedIds);
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptySet();
        }
        return ids.stream().filter(Objects::nonNull).collect(Collectors.toSet());
    }

    /**
     * 统计的打赏值Map
     *
     * @param feedIds
     * @return feedId:打赏总量
     */
    public Map<Long, Long> statRewardMap(Collection<Long> feedIds) {
        if (CollectionUtils.isEmpty(feedIds)) {
            return Collections.emptyMap();
        }

        List<StatRewardFeedGiftDO> list = statRewardFeedGiftDAO.selectTotalReward(StatTimeType.YEAR, feedIds,
                StatRewardFeedGiftTypeEnum.SEND_STAR_DIAMOND.getValue());
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }

        return list.stream()
                .filter(Objects::nonNull)
                .filter(po -> po.getBizId() != null)
                .collect(Collectors.toMap(StatRewardFeedGiftDO::getBizId, StatRewardFeedGiftDO::getStatValue));
    }

    /**
     * 获得打赏数量展示值
     *
     * @param total
     * @return
     */
    public String getRewardStr(Long total) {
        if (total == null) {
            return "";
        }
        String value = "" + total;
        if (total > 9999) {
            BigDecimal showTotal = new BigDecimal(total + "")
                    .divide(new BigDecimal("10000"), 2, RoundingMode.DOWN);
            value = showTotal.toPlainString() + "w";
        }
        return value;
    }

}
