package com.starbuds.server.service.feed.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.feed.FeedCreateBusinessEnum;
import com.starbuds.server.common.constant.feed.FeedStatusEnum;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.FeedDO;
import com.starbuds.server.common.pojo.db.FeedPhotoWallDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataGreenService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.service.feed.dao.FeedDAO;
import com.starbuds.server.service.feed.dao.FeedPhotoWallDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

import static com.starbuds.server.common.constant.data.GreenAuditStatusEnum.PASSED_ROBOT;
import static com.starbuds.server.common.constant.feed.FeedCreateBusinessEnum.PHOTO_WALL;
import static com.starbuds.server.common.constant.feed.FeedStatusEnum.PASSED;
import static com.starbuds.server.service.feed.util.SpringContextUtil.isDev;

/**
 * 同步图片风险检测
 *
 * @author aoe
 * @date 2020/5/28
 */
@Slf4j
@Component
public class ImageSyncScanProvider {

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private FeedDAO feedDAO;

    @Resource
    private FeedPhotoWallDAO feedPhotoWallDAO;

    @Reference(version = "1.0.0")
    private StatPlatformService statPlatformService;

    @Reference(version = "1.0.0", timeout = 10000, retries = 0)
    private DataGreenService dataGreenService;

    private Map<GreenAuditStatusEnum, FeedStatusEnum> statusMap;

    @PostConstruct
    private void init() {
        statusMap = new HashMap<>();
        statusMap.put(PASSED_ROBOT, PASSED);
        statusMap.put(GreenAuditStatusEnum.BLOCK_ROBOT, FeedStatusEnum.REFUSE_ROBOT);
        statusMap.put(GreenAuditStatusEnum.REVIEW_ROBOT, FeedStatusEnum.REVIEW_ROBOT);
        statusMap.put(GreenAuditStatusEnum.BLOCK_EXCEPTION, FeedStatusEnum.TIMEOUT_ROBOT);
        statusMap.put(GreenAuditStatusEnum.BLOCK_TIMEOUT, FeedStatusEnum.TIMEOUT_ROBOT);
    }

    /**
     * 审核图片内容
     *
     * @param feedId
     * @param url
     */
    public void scanImage(long feedId, String url, Long channelId, FeedCreateBusinessEnum createBusinessEnum) {
        log.info("开始内容审核");

        GreenAuditTrace greenAuditTrace = dataGreenService.imgAudit(url, Long.toString(feedId), feedId,
                PHOTO_WALL == createBusinessEnum ? GreenBizTypeEnum.PHOTO_WALL : GreenBizTypeEnum.FEED_IMAGE);
        // 返回审核中/失败
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        String auditMsg = robotAuditStatus == PASSED_ROBOT ? "通过" : greenAuditTrace.getFailMsg();

        FeedStatusEnum statusEnum = statusMap.get(robotAuditStatus);
        if (statusEnum == null) {
            statusEnum = FeedStatusEnum.TIMEOUT_ROBOT;
        }
        //if (isDev()) {
        log.info("审核图片内容结果 feedId={} url={} statusEnum={}", feedId, url, statusEnum);
        //}
        if (PHOTO_WALL == createBusinessEnum) {
            // 照片墙
            updatePhotoWallAuditResult(feedId, statusEnum, auditMsg);
        } else {
            // 动态
            updateAuditResult(feedId, statusEnum, auditMsg);
        }

        // 平台数据统计服务
        if (statusEnum == PASSED) {
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.FEED_AUDIT_AUTO_PASS, BigDecimal.ONE);
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.PHOTO_AUDIT_PASS, BigDecimal.ONE);
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.PHOTO_AUDIT_AUTO_PASS, BigDecimal.ONE);
        }
    }

    /**
     * 更新审核结果
     *
     * @param feedId
     * @param statusEnum
     * @param auditMsg
     */
    private void updateAuditResult(Long feedId, FeedStatusEnum statusEnum, String auditMsg) {
        FeedDO feed = new FeedDO();
        feed.setFeedId(feedId);
        feed.setFeedStatus(statusEnum.value());
        feed.setAuditMsg(auditMsg);
        feed.setUpdateTime(System.currentTimeMillis());
        int result = feedDAO.update(feed);
        log.info("存储图片内容审核更新数据库 result={} feedId={} status={} auditMsg={}", result, feedId, statusEnum.value(), auditMsg);
    }

    /**
     * 更新照片墙审核结果
     *
     * @param feedPhotoWallId
     * @param statusEnum
     * @param auditMsg
     */
    private void updatePhotoWallAuditResult(Long feedPhotoWallId, FeedStatusEnum statusEnum, String auditMsg) {
        FeedPhotoWallDO fpw = new FeedPhotoWallDO();
        fpw.setFeedPhotoWallId(feedPhotoWallId);
        fpw.setStatus(statusEnum.value());
        fpw.setAuditMsg(auditMsg);
        fpw.setUpdateTime(System.currentTimeMillis());
        int result = feedPhotoWallDAO.update(fpw);
        log.info("存储图片内容审核更新数据库 result={} feedPhotoWallId={} status={} auditMsg={}", result, feedPhotoWallId, statusEnum.value(), auditMsg);
    }
}
